#include "eventpasswidget.h"

#include <QTimer>
#include <QDebug>
#include <QMouseEvent>
#include <QTouchEvent>

#include <ukui-log4qt.h>

#include "global/globalsignal.h"
#include "global/global.h"

using namespace Global;

EventPassWidget::EventPassWidget(QWidget *parent) : QWidget(parent)
{
    setAttribute(Qt::WA_TranslucentBackground);
    setAttribute(Qt::WA_AcceptTouchEvents);

    setMouseTracking(true);

    m_mouseUsed = true;

    initGlobalSig();

    m_checkMouseTimer = new QTimer;
    m_checkMouseTimer->setInterval(2000);
    connect(m_checkMouseTimer, &QTimer::timeout, [&](){
        if(!m_hasVideo)
            return;
        if(m_isMouseEnter) {
            if (m_mouseUsed)
                setCursor(QCursor(Qt::BlankCursor));
        }
        g_user_signal->hideBar(true);

        m_checkMouseTimer->stop();
        return;
    });

    m_lMouseClickTimer = new QTimer;
    m_lMouseClickTimer->setInterval(300);
    connect(m_lMouseClickTimer, &QTimer::timeout, [&](){
        emit mousePressed();
        m_lMouseClickTimer->stop();
    });
}

void EventPassWidget::setMouseUsed(bool used)
{
    m_mouseUsed = used;
    if (!m_mouseUsed) {
        setCursor(QCursor(Qt::ArrowCursor));
        m_checkMouseTimer->stop();
    }
}

void EventPassWidget::playStateChange(Mpv::PlayState state)
{
    m_playState = state;
    if(m_playState < 0) {
        m_checkMouseTimer->stop();
    }
    else {
        m_checkMouseTimer->start();
    }
}

void EventPassWidget::videIdChange(int vid)
{
    if(vid < 0)
        m_hasVideo = false;
    else
        m_hasVideo = true;
}

void EventPassWidget::initGlobalSig()
{
    connect(g_core_signal, &GlobalCoreSignal::sigStateChange, this, &EventPassWidget::playStateChange);
    connect(g_core_signal, &GlobalCoreSignal::sigVideoIdChange, this, &EventPassWidget::videIdChange);
}

void EventPassWidget::mousePressEvent(QMouseEvent *e)
{
    m_isMousePressed = true;
    m_mousePosPressed = e->pos();
    m_posStart = e->pos();
    m_posLast = e->pos();
}

void EventPassWidget::mouseReleaseEvent(QMouseEvent *e)
{
    m_isMousePressed = false;
    KyInfo() << e->button() << m_mousePosPressed << e->pos();
    if (e->button() == Qt::RightButton) {
        m_isMouseEnter = false;
        g_user_signal->showRightMenu();
    }
    else if (e->button() == Qt::LeftButton) {
        if (!m_mouseUsed)
            return;
        if (e->pos() == m_mousePosPressed)
            m_lMouseClickTimer->start();
    }
}

void EventPassWidget::mouseMoveEvent(QMouseEvent *e)
{
    if (m_isMousePressed) {
        m_posCurrent = e->pos();

        int xc = abs(m_posCurrent.x() - m_posLast.x());
        int yc = abs(m_posCurrent.y() - m_posLast.y());
        if (xc < yc) {
            m_volumeChange = yc * 200 / height();
            //符合音量调节
            if (m_volumeChange > 0) {
                if (m_posCurrent.y() - m_posLast.y() > 2) {
                    g_user_signal->setVolumeDown(m_volumeChange);
                    m_posLast = m_posCurrent;
                }
                else {
                    g_user_signal->setVolumeUp(m_volumeChange);
                    m_posLast = m_posCurrent;
                }
            }
        }
        return QWidget::mouseMoveEvent(e);
    }

    m_isMouseEnter = true;
    m_checkMouseTimer->stop();
    if (cursor().shape() != Qt::ArrowCursor) {
        setCursor(QCursor(Qt::ArrowCursor));
    }
    g_user_signal->hideBar(false);
    if(m_playState > 0)
        m_checkMouseTimer->start();
}

void EventPassWidget::mouseDoubleClickEvent(QMouseEvent *e)
{
    if (!m_mouseUsed)
        return;
    if(e->buttons() == Qt::LeftButton) {
        g_user_signal->fullScreen();
        m_lMouseClickTimer->stop();
    }
}

void EventPassWidget::enterEvent(QEvent *e)
{
    m_isMouseEnter = true;
}

void EventPassWidget::leaveEvent(QEvent *e)
{
    m_isMouseEnter = false;
    m_isMousePressed = false;
    if(m_playState > 0)
        m_checkMouseTimer->start();
}

bool EventPassWidget::event(QEvent *e)
{
#if 0
    QTouchEvent* touch = nullptr;
    switch(e->type()) {
    case QEvent::TouchBegin:
        touch = static_cast<QTouchEvent*>(e);
        if (touch->touchPoints().size() == 1) {
            m_posStart = touch->touchPoints().at(0).pos().toPoint();
            m_posLast = touch->touchPoints().at(0).pos().toPoint();
        }
        m_volumeChange = 0;
        break;
    case QEvent::TouchUpdate:
        touch = static_cast<QTouchEvent*>(e);
        if (touch->touchPoints().size() == 1) {
            m_posCurrent = touch->touchPoints().at(0).pos().toPoint();

            int xc = abs(m_posCurrent.x() - m_posLast.x());
            int yc = abs(m_posCurrent.y() - m_posLast.y());
            if (xc < yc) {
                m_volumeChange = yc * 100 / height();
                //符合音量调节
                if (m_posCurrent.y() - m_posLast.y() > 2) {
                    g_user_signal->setVolumeUp(m_volumeChange);
                }
                else {
                    g_user_signal->setVolumeDown(m_volumeChange);
                }
            }
            m_posLast = m_posCurrent;
        }
        break;
    case QEvent::TouchEnd:
        touch = static_cast<QTouchEvent*>(e);
        if (touch->touchPoints().size() == 1) {

        }
        break;
    default:
        break;
    }
#endif
    return QWidget::event(e);
}
